//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2015 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Filename      : $RCSfile: N_ANP_SecondLevelManager.h,v $
//
// Purpose       :
//
// Special Notes :
//
// Creator       : Richard Schiek, SNL, Electrical and Microsystem Modeling
//
// Creation Date : 01/24/08
//
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.2.2.1 $
//
// Revision Date  : $Date: 2015/04/02 18:20:06 $
//
// Current Owner  : $Author: tvrusso $
//-----------------------------------------------------------------------------

#ifndef Xyce_N_ANP_SecondLevelManager_h
#define Xyce_N_ANP_SecondLevelManager_h

#include <N_ANP_AnalysisManager.h>

namespace Xyce {
namespace Analysis {

class SecondLevelManager : public AnalysisManager
{
public:
  SecondLevelManager(
    const IO::CmdParse &        command_line,
    IO::RestartMgr &            restart_manager,
    OutputMgrAdapter &          output_manager_adapter,
    Stats::Stat                 analysis_stat)
    : AnalysisManager(command_line, restart_manager, output_manager_adapter, analysis_stat)
  {}

  virtual ~SecondLevelManager()
  {}

private:
  SecondLevelManager(const SecondLevelManager &);
  SecondLevelManager &operator=(const SecondLevelManager &);

public:
  // Two-level Newton API functions:
  // Execute the control loop for the set analysis type,
  // for a set number of steps.
  void setExternalSolverState (const Device::SolverState & ss);
  bool runSecondLevelStep(const TimeIntg::TimeIntInfo & tiInfo, TimeIntg::TwoLevelError & tlError);

  bool startupSecondLevelSolvers(Linear::System &linear_system, Nonlinear::Manager &nonlinear_manager);
  bool finishSecondLevelSolvers();

  void homotopyStepSuccess(const std::vector<std::string> & paramNames, const std::vector<double> & paramVals);
  void homotopyStepFailure();

    void stepSecondLevelSuccess(TwoLevelMode analysisUpper);
    void stepSecondLevelFailure(TwoLevelMode analysisUpper);
    bool getSecondLevelInitialQnorm (TimeIntg::TwoLevelError & tle) const;
    bool getSecondLevelBreakPoints(std::vector<Util::BreakPoint> &breakPointTimes) const;
    bool startSecondLevelTimeStep(const TimeIntg::TIAParams &tia_params, Nonlinear::Manager &nonlinear_manager, const TimeIntg::TimeIntInfo & tiInfo);

private:
  AnalysisBase *        twoLevelAnalysisObject_;
  IO::ActiveOutput *    activeOutput_;
};

} // namespace Analysis
} // namespace Xyce

#endif // Xyce_N_ANP_SecondLevelManager_h
