//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2015 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Filename       : $RCSfile: N_ANP_Step.h,v $
//
// Purpose        : Step analysis class
//
// Special Notes  : Specify any "hidden" or subtle details of the class here.
//                  Portability details, error handling information, etc.
//
// Creator        : Richard Schiek, SNL, Electrical and Microsystem Modeling
//
// Creation Date  : 01/24/08
//
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.31.2.1 $
//
// Revision Date  : $Date: 2015/04/02 18:20:06 $
//
// Current Owner  : $Author: tvrusso $
//-----------------------------------------------------------------------------

#ifndef Xyce_N_ANP_Step_h
#define Xyce_N_ANP_Step_h

#include <N_ANP_fwd.h>

#include <N_ANP_AnalysisBase.h>
#include <N_ANP_Factory.h>

namespace Xyce {
namespace Analysis {

//-------------------------------------------------------------------------
// Class         : Step
// Purpose       : Step analysis class
// Special Notes :
// Creator       : Richard Schiek, SNL, Electrical and Microsystem Modeling
// Creation Date : 01/24/08
//-------------------------------------------------------------------------
class Step : public AnalysisBase
{
public:
  Step(AnalysisManager &analysis_manager, AnalysisBase &child_analysis)
    : AnalysisBase(analysis_manager, "Step"),
      analysisManager_(analysis_manager),
      loader_(analysis_manager.getLoader()),
      outputManagerAdapter_(analysis_manager.getOutputManagerAdapter()),
      childAnalysis_(child_analysis),
      stepSweepVector_(),
      stepLoopSize_(0)
  {}

  virtual ~Step()
  {}

  bool setAnalysisParams(const Util::OptionBlock & paramsBlock);

  const TimeIntg::TIAParams &getTIAParams() const; // override
  TimeIntg::TIAParams &getTIAParams(); // override

  virtual bool getDCOPFlag() const;

  virtual bool doRun();
  virtual bool doInit();
  virtual bool doLoopProcess();
  virtual bool doProcessSuccessfulStep();
  virtual bool doProcessFailedStep();
  virtual bool doFinish();
  virtual bool doHandlePredictor() { return true; }

private:
  AnalysisManager &     analysisManager_;
  Loader::Loader &      loader_;
  OutputMgrAdapter &    outputManagerAdapter_;
  AnalysisBase &        childAnalysis_;
  SweepVector           stepSweepVector_;
  int                   stepLoopSize_;
};


bool registerStepFactory(
  const std::string &   netlist_filename,
  IO::PkgOptionsMgr &   options_manager,
  AnalysisManager &     analysis_manager,
  Linear::System &      linear_system,
  Nonlinear::Manager &  nonlinear_manager);

} // namespace Analysis
} // namespace Xyce

#endif // Xyce_N_ANP_Step_h
