//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2015 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
// Filename       : $RCSfile: N_IO_LoadManager.C,v $
//
// Purpose        : Output Manager
//
// Special Notes  :
//
// Creator        : Robert J. Hoekstra, SNL, Parallel Computational Sciences
//
// Creation Date  : 10/10/00
//
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.3.2.1 $
//
// Revision Date  : $Date: 2015/04/02 18:20:14 $
//
// Current Owner  : $Author: tvrusso $
//-------------------------------------------------------------------------

#include <Xyce_config.h>

#include <N_IO_LoadManager.h>
#include <N_IO_PkgOptionsMgr.h>
#include <N_ERH_Message.h>

namespace Xyce {
namespace IO {

namespace {

//-----------------------------------------------------------------------------
// Class         : LoadOptionsRegx
// Purpose       : functor for registering Load options
// Special Notes : Used by package manager submitRegistration method
// Creator       : dgbaur
// Creation Date : 06/28/2013
//-----------------------------------------------------------------------------
struct LoadOptionsReg : public PkgOptionsReg
{
  LoadOptionsReg()
  {}

  bool operator()( const Util::OptionBlock & options )
  {
    Report::UserWarning0() << ".LOAD not supported yet.  Use .INCLUDE instead";
    return true;
  }
};

} // namespace <unnamed>

//-----------------------------------------------------------------------------
// Function      : registerPkgOptionsMgr
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : Rich Schiek, 1437
// Creation Date : 10/21/08
//-----------------------------------------------------------------------------
bool registerPkgOptionsMgr(LoadManager &load, const std::string &netlist_filename, PkgOptionsMgr &options_manager)
{
  options_manager.submitRegistration(
    "LOAD", netlist_filename, new LoadOptionsReg());

  return true;
}

} // namespace IO
} // namespace Xyce
