//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2015 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Filename       : $RCSfile: N_LAS_PrecondFactory.h,v $
//
// Purpose        : Preconditioner Factory
//
// Special Notes  :
//
// Creator        : Heidi Thornquist, SNL, Electrical & Microsystem Modeling
//
// Creation Date  : 10/01/07
//
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.12.2.1 $
//
// Revision Date  : $Date: 2015/04/02 18:20:15 $
//
// Current Owner  : $Author: tvrusso $
//-----------------------------------------------------------------------------

#ifndef Xyce_N_LAS_PrecondFactory_h
#define Xyce_N_LAS_PrecondFactory_h

#include <string>

#include <N_LAS_fwd.h>
#include <N_UTL_fwd.h>

#include <Teuchos_RCP.hpp>

namespace Xyce {
namespace Linear {

//-----------------------------------------------------------------------------
// Class         : PrecondFactory
// Purpose       :
// Special Notes :
// Creator       : Heidi Thornquist, SNL, Electrical & Microsystem Modeling
// Creation Date : 10/01/07
//-----------------------------------------------------------------------------
class PrecondFactory
{
public:
  // Default Constructor
  PrecondFactory() {}

  // Basic Constructor, sets preconditioner factory options.
  PrecondFactory( const Util::OptionBlock & OB ) {}

  // Destructor
  virtual ~PrecondFactory() {}

  // Creates a new preconditioner (matrix based).
  virtual Teuchos::RCP<Preconditioner> create( const Teuchos::RCP<Problem> & problem ) const = 0;

  // Creates a new preconditioner (matrix free).
  virtual Teuchos::RCP<Preconditioner> create( const Teuchos::RCP<System> & lasSystem ) const = 0;
};

} // namespace Linear
} // namespace Xyce

#endif // Xyce_N_LAS_PrecondFactory_h
