//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2015 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
// Filename       : $RCSfile: N_LAS_QueryUtil.C,v $
//
// Purpose        :
//
// Special Notes  :
//
// Creator        : Robert J. Hoekstra, SNL, Parallel Computational Sciences
//
// Creation Date  : 05/26/00
//
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.2.2.1 $
//
// Revision Date  : $Date: 2015/04/02 18:20:15 $
//
// Current Owner  : $Author: tvrusso $
//-------------------------------------------------------------------------

#include <Xyce_config.h>

#include <iostream>
#include <numeric>
#include <assert.h>

#include <N_ERH_ErrorMgr.h>
#include <N_IO_PkgOptionsMgr.h>
#include <N_LAS_QueryUtil.h>
#include <N_UTL_OptionBlock.h>

namespace Xyce {
namespace Linear {

namespace {

struct OptionsReg : public IO::PkgOptionsReg
{
  OptionsReg( QueryUtil & lsu )
    : lsUtil(lsu)
  {}

  bool operator()( const Util::OptionBlock & options )
  { return lsUtil.registerOptions( options ); }

  QueryUtil & lsUtil;
};

struct TimeOptionsReg : public IO::PkgOptionsReg
{
  TimeOptionsReg( QueryUtil & lsu )
    : lsUtil(lsu)
  {}

  bool operator()( const Util::OptionBlock & options )
  { return lsUtil.registerTimeOptions( options ); }

  QueryUtil & lsUtil;
};

} // namespace <unnamed>

//-----------------------------------------------------------------------------
// Function      : QueryUtil::registerPkgOptionsMgr
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : Rich Schiek, 1437
// Creation Date : 10/21/08
//-----------------------------------------------------------------------------
bool registerPkgOptionsMgr(QueryUtil &linear_solve_utility, const std::string &netlist_filename, IO::PkgOptionsMgr &options_manager)
{
  options_manager.submitRegistration( "TIMEINT", netlist_filename, new TimeOptionsReg(linear_solve_utility));
  options_manager.submitRegistration( "TOPOLOGY", netlist_filename, new OptionsReg(linear_solve_utility));

  return true;
}


//-----------------------------------------------------------------------------
// Function      : QueryUtil::QueryUtil
// Purpose       : constructor
// Special Notes :
// Scope         : public
// Creator       : Eric Keiter  SNL, Parallel Computational Sciences
// Creation Date : 5/25/05
//-----------------------------------------------------------------------------
QueryUtil::QueryUtil()
  : checkConnectivity_(true),
    supernode_(false),
#ifdef Xyce_TEST_SOLN_VAR_MAP
    namesFile_(true)
#else
    namesFile_(false)
#endif
{
}

//-----------------------------------------------------------------------------
// Function      : QueryUtil::registerOptions
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : Robert Hoekstra, SNL, Parallel Computational Sciences
// Creation Date : 6/20/05
//-----------------------------------------------------------------------------
bool QueryUtil::registerOptions(const Util::OptionBlock & OB)
{
  Util::ParamList::const_iterator it_tpL;
  Util::ParamList::const_iterator first = OB.begin();
  Util::ParamList::const_iterator last  = OB.end();

  for (it_tpL = first; it_tpL != last; ++it_tpL)
  {
    if (it_tpL->uTag()=="CHECK_CONNECTIVITY")
    {
      checkConnectivity_ = static_cast<bool> (it_tpL->getImmutableValue<int>());
    }
    else if(it_tpL->uTag()=="SUPERNODE")
    {
      supernode_ = static_cast<bool>(it_tpL->getImmutableValue<bool>());
    }
    else if(it_tpL->uTag()=="OUTPUTNAMESFILE")
    {
      namesFile_ = static_cast<bool>(it_tpL->getImmutableValue<bool>());
    }
  }

  return true;
}

} // namespace Linear
} // namespace Xyce
