//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2015 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Filename      : $RCSfile: N_TIA_TimeIntInfo.h,v $
// Purpose       :
// Special Notes :
// Creator       : Eric Keiter
// Creation Date : 1/29/07
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.22.2.1 $
//
// Revision Date  : $Date: 2015/04/02 18:20:18 $
//
// Current Owner  : $Author: tvrusso $
//-----------------------------------------------------------------------------

#ifndef Xyce_N_TIA_TimeIntInfo_h
#define Xyce_N_TIA_TimeIntInfo_h

// ---------- Standard Declarations ----------
#include <iosfwd>

// ---------- Forward Declarations ----------


namespace Xyce {
namespace TimeIntg {

//-----------------------------------------------------------------------------
// Class         : TimeIntInfo
//
// Purpose       : This class contains time integration info that needs to
//                 be passed from the time integration package to other
//                 parts of Xyce.
//
// Special Notes :
// Creator       : Eric Keiter, SNL
// Creation Date : 1/29/07
//-----------------------------------------------------------------------------
struct TimeIntInfo
{
  TimeIntInfo()
    : currentOrder(1),
      numberOfSteps(0),
      usedOrder(1),
      nscsco(0),
      pdt(0.0),
      nextTimeStep(0.0),
      currTimeStep(0.0),
      currentTime(0.0),
      nextTime(0.0),
      finalTime(0.0),
      startingTimeStep(0.0),
      bpTol(0.0),
      dcopFlag(false),
      acopFlag(false),
      inputOPFlag(false),
      tranopFlag(false),
      transientFlag(false),
      dcsweepFlag(false),
      timeStepNumber(0),
      initTranFlag(false),
      beginIntegrationFlag(false),
      doubleDCOPStep(0),
      doubleDCOPEnabled(false),
      timeIntMode(0),
      sweepSourceResetFlag(false)
  {}

  ~TimeIntInfo()
  {}

  int           currentOrder;           ///< Time integrator order
  int           numberOfSteps;          ///< Time integrator step
  int           usedOrder;              ///< Time integrator used order
  int           nscsco;                 ///< Time integrator ?nscsco?
  double        pdt;                    ///< Time integrator partial time deriviative

  double        nextTimeStep;           ///< Step error control next time step
  double        currTimeStep;           ///< Step error control current time step
  double        currentTime;            ///< Step error control current time
  double        nextTime;               ///< Step error control next time
  double        finalTime;              ///< Step error control final time
  double        startingTimeStep;       ///< Step error control starting time step
  double        bpTol;                  ///< Step error control breakpoint tolerance

  bool          dcopFlag;               ///< Analysis DCOP flag
  bool          acopFlag;               ///< Analysis ACOP flag
  bool          inputOPFlag;            ///< Analysis InputOP flag
  bool          tranopFlag;             ///< Analysis TransientOP flag
  bool          transientFlag;          ///< Analysis
  bool          dcsweepFlag;            ///< Analysis
  bool          sweepSourceResetFlag;   ///< Analysis
  int           timeStepNumber;         ///< Analysis
  bool          initTranFlag;           ///< Analysis
  bool          beginIntegrationFlag;   ///< Analysis
  int           doubleDCOPStep;         ///< Analysis
  bool          doubleDCOPEnabled;      ///< Analysis
  int           timeIntMode;            ///< Analysis
};

//-----------------------------------------------------------------------------
// Function      : operator<<
// Purpose       : "<<" operator for two-level info class.
// Special Notes :
// Scope         : public
// Creator       : Eric R. Keiter, SNL, Parallel Computational Sciences
// Creation Date : 1/29/07
//-----------------------------------------------------------------------------
std::ostream & operator<<(std::ostream & os, const TimeIntInfo & tiInfo);

} // namespace TimeIntg
} // namespace Xyce

#endif // Xyce_N_TIA_TimeIntInfo_h

