//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2015 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Filename       : $RCSfile: N_TOP_NodeDevBlock.h,v $
//
// Purpose        : Block of Node and Dev data for passing between pkgs
//
// Special Notes  :
//
// Creator        : Robert J. Hoekstra, SNL, Parallel Computational Sciences
//
// Creation Date  : 3/3/01
//
// Revision Information:
// ---------------------
//
// Revision Number: $Revision: 1.23.2.1 $
//
// Revision Date  : $Date: 2015/04/02 18:20:18 $
//
// Current Owner  : $Author: tvrusso $
//-----------------------------------------------------------------------------

#ifndef N_TOP_NodeDevBlock_h
#define N_TOP_NodeDevBlock_h 1

#include <iosfwd>

#include <N_UTL_Packable.h>
#include <N_DEV_DeviceBlock.h>
#include <N_TOP_Misc.h>

namespace Xyce {
namespace Topo {

//-----------------------------------------------------------------------------
// Class         : NodeDevBlock
// Purpose       :
// Special Notes :
// Creator       : Rob Hoekstra, SNL, Parallel Computational Sciences
// Creation Date : 3/3/01
//-----------------------------------------------------------------------------
class NodeDevBlock : public Packable
{
  friend std::ostream &operator<<(std::ostream & os, const NodeDevBlock & ndb);

public:
  NodeDevBlock(
    const Device::InstanceBlock &       ib = Device::InstanceBlock())
  : devBlock_(ib)
  {}

  NodeDevBlock(const NodeDevBlock & right)
  : devBlock_(right.devBlock_)
  {}

  ~NodeDevBlock()
  {}

private:
  // Assignment operator (autogen)
  NodeDevBlock & operator = (const NodeDevBlock & right);

public:
  // clear data for reuse of this object
  void clear();

  void addNode(const tagged_param &);
  const std::list<tagged_param> & get_NodeList() const;
  void set_NodeList(const std::list<tagged_param> & nList);

  Device::InstanceBlock & getDevBlock() { return devBlock_; }
  const Device::InstanceBlock & getDevBlock() const { return devBlock_; }

  bool isDevice() const { return devBlock_.getInstanceName() != ""; }

  // For packing and unpacking using MPI.
  Packable * instance() const;

  // Counts number of bytes needed to pack object.
  int packedByteCount() const;

  // Packs NodeList into char buffer using MPI_PACK.
  void pack(char * buf, int bsize, int & pos, N_PDS_Comm * comm) const;
  // Unpacks NodeList from char buffer using MPI_UNPACK.
  void unpack(char * pB, int bsize, int & pos, N_PDS_Comm * comm);

protected:

  std::list<tagged_param>       nodeList_;
  Device::InstanceBlock         devBlock_;
};

} // namespace Topo
} // namespace Xyce

#endif
